
package body Vectors.IO is

  package Txt_Int_IO is new Ada.Text_IO.Integer_IO(Integer);
  use Txt_Int_IO;

  function Width(I: Integer; Base: Positive := 10) return Positive is
    K,D: Integer;
  begin
    K := abs(I);
    if I<0 then D := 2; else D := 1; end if;
    while K >= Base loop
      K := K/Base;
      D := D+1;
    end loop;
    return D;
  end Width;

  procedure Show(V: in Vector) is
    W: constant Positive := 3;
  begin
    for I in V'Range loop
      if not IsZero(V(I)) then
        Put(Current_Output,I,W);
        Put(" :  ");
        Put(Current_Output,V(I));
        New_Line;
      end if;
    end loop;
    New_Line;
  end Show;

  procedure Get(F: in File_Type; V: out Vector; Decimal: in Boolean := True) is
    Trunc: Boolean := False;
    I,L: Integer;
    S: Scalar;
  begin
    SetZero(V);
    Get(F,L);
    I := V'First;
    for M in 1 .. L loop
       Get(F,S,Decimal);
       if I <= V'Last then V(I) := S; else Trunc := True; end if;
       I := I+1;
    end loop;
    if Trunc then
       Put_Line("Vectors.IO.Get Warning: input truncated");
    end if;
  end Get;

  function EffLast(V: Vector) return Integer is
  begin
    for I in reverse V'First+1 .. V'Last loop
      if not IsZero(V(I)) then return I; end if;
    end loop;
    return V'First;
  end EffLast;

  procedure Put(F: in File_Type; V: in Vector; Decimal: in Boolean := True) is
    VLast: constant Integer := EffLast(V);
    VLength: constant Integer := VLast-V'First+1;
  begin
     Put(F,VLength,Width(VLength));
     New_Line(F);
     for I in V'First .. VLast loop
        Put(F,V(I),Decimal);
        New_Line(F);
     end loop;
  end Put;

  procedure Show(A: in Matrix) is
     W: constant Positive := 3;
  begin
     for I in A'Range(1) loop
        for J in A'Range(2) loop
           if not IsZero(A(I,j)) then
              Put(Current_Output,I,W);
              Put(Current_Output,J,W);
              Put(" :  ");
              Put(Current_Output,A(I,J));
              New_Line;
           end if;
        end loop;
        New_Line;
     end loop;
  end Show;

  procedure Get(F: in File_Type; A: out Matrix; Decimal: in Boolean := True) is
    Trunc: Boolean := False;
    I,J,LM,LN: Integer;
    S: Scalar;
  begin
    SetZero(A);
    Get(F,LM);
    Get(F,LN);
    I := A'First(1);
    for M in 1 .. LM loop
      J := A'First(2);
      for N in 1 .. LN loop
        Get(F,S,Decimal);
        if I <= A'Last(1) and then J <= A'Last(2) then A(I,J) := S; else Trunc := True; end if;
        J := J+1;
      end loop;
      I := I+1;
    end loop;
    if Trunc then
      Put_Line("Vectors.IO.Get Warning: input truncated");
    end if;
  end Get;

  function EffLast1(A: Matrix) return Integer is
  begin
    for I in reverse A'First(1)+1 .. A'Last(1) loop
      for J in A'Range(2) loop
        if not IsZero(A(I,J)) then return I; end if;
      end loop;
    end loop;
    return A'First(1);
  end EffLast1;

  function EffLast2(A: Matrix; ALast1: Integer) return Integer is
    L: Integer := A'First(2);
  begin
    for I in A'First(1) .. ALast1 loop
      for J in reverse L+1 .. A'Last(2) loop
        if not IsZero(A(I,J)) then L := J; exit; end if;
      end loop;
    end loop;
    return L;
  end EffLast2;

  procedure Put(F: in File_Type; A: in Matrix; Decimal: in Boolean := True) is
    ALast1: constant Integer := EffLast1(A);
    ALast2: constant Integer := EffLast2(A,ALast1);
    ALength1: constant Integer := ALast1-A'First(1)+1;
    ALength2: constant Integer := ALast2-A'First(2)+1;
  begin
     Put(F,ALength1,Width(ALength1));
     New_Line(F);
     Put(F,ALength2,Width(ALength2));
     New_Line(F);
     for I in A'First(1) .. ALast1 loop
        for J in A'First(2) .. ALast2 loop
           Put(F,A(I,J),Decimal);
           New_Line(F);
        end loop;
     end loop;
  end Put;

end Vectors.IO;
